from __future__ import annotations

from pathlib import Path

from .helpers import run


def test_history(request):
    readme_path = Path(request.config.rootdir, 'README.md')
    cmd = ['history', '--baseline-path', str(readme_path), '--no-color']
    actual = run(cmd)
    exp = '2022-09-01 11:45:28+02:00  60 =   3   -1  +58   8fe7afd10c  git@orsinium.dev'
    assert exp in actual



from __future__ import annotations

from io import StringIO

from mypy_baseline import main


LINE1 = 'views.py:69: error: Hello world  [assignment]\r\n'
LINE2 = 'settings.py:42: error: How are you?  [union-attr]\r\n'
LINE3 = 'python/utils.py:15: error: Second argument of Enum() must be string  [misc]\n'
LINE_WITH_NOTE = 'integrations/services.py:0: note: See https://mypy.readthedocs.io/en/stable/running_mypy.html#missing-imports'  # noqa: E501
SUCCESS_LINE = 'Success: no issues found in 26 source files'
NOTEBOOK_LINE1 = 'fail.ipynb:cell_1:2: \x1b[1m\x1b[31merror:\x1b(B\x1b[m Incompatible return value type (got \x1b(B\x1b[m\x1b[1m"int"\x1b(B\x1b[m, expected \x1b(B\x1b[m\x1b[1m"str"\x1b(B\x1b[m)  \x1b(B\x1b[m\x1b[33m[return-value]\x1b(B\x1b[m\n'  # noqa: E501


def run(cmd: list[str], exit_code: int = 0) -> str:
    stdout = StringIO()
    code = main(cmd, StringIO(), stdout)
    stdout.seek(0)
    result = stdout.read()
    assert code == exit_code, result
    return result



from __future__ import annotations

from io import StringIO

from mypy_baseline import main

from .helpers import (
    LINE1, LINE2, LINE3, LINE_WITH_NOTE, NOTEBOOK_LINE1, SUCCESS_LINE, run,
)


def test_filter():
    stdin = StringIO()
    stdin.write(LINE1)
    stdin.write(LINE2)
    stdin.write(LINE3)
    stdin.write(LINE_WITH_NOTE)
    stdin.seek(0)
    stdout = StringIO()
    code = main(['filter'], stdin, stdout)
    assert code == 4
    actual = stdout.getvalue()
    assert LINE1.strip() in actual
    assert LINE2.strip() in actual
    assert LINE3.strip() in actual
    assert LINE_WITH_NOTE.strip() in actual
    assert '  assignment  ' in actual
    assert '  union-attr  ' in actual
    assert '  unresolved' in actual
    assert ' note ' in actual
    assert 'Your changes introduced' in actual


def test_filter__ignore_category():
    stdin = StringIO()
    stdin.write(LINE_WITH_NOTE)
    stdin.seek(0)
    stdout = StringIO()
    code = main(['filter', '--ignore-categories', 'note'], stdin, stdout)
    assert code == 0
    actual = stdout.getvalue()
    assert actual == ''


def test_filter__ignore():
    stdin = StringIO()
    stdin.write(LINE3)
    stdin.seek(0)
    stdout = StringIO()
    code = main(['filter', '--ignore', '.*Enum.*'], stdin, stdout)
    assert code == 0
    actual = stdout.getvalue()
    assert actual == ''


def test_filter_notebook():
    stdin = StringIO()
    stdin.write(NOTEBOOK_LINE1)

    stdin.seek(0)
    stdout = StringIO()
    code = main(['filter'], stdin, stdout)
    assert code == 1
    actual = stdout.getvalue()
    assert NOTEBOOK_LINE1 in actual
    assert '  return-value  ' in actual
    assert '  unresolved' in actual
    assert 'Your changes introduced' in actual


def test_filter__empty_stdin():
    actual = run(['filter'])
    assert actual == ''


def test_filter_success():
    stdin = StringIO()
    stdin.write(SUCCESS_LINE)
    stdin.seek(0)
    stdout = StringIO()
    code = main(['filter'], stdin, stdout)
    assert code == 0
    actual = stdout.getvalue()
    assert actual == SUCCESS_LINE



from __future__ import annotations

from io import StringIO
from pathlib import Path

from mypy_baseline import main

from .helpers import LINE1, LINE2, LINE3, NOTEBOOK_LINE1


def test_sync(tmp_path: Path):
    blpath = tmp_path / 'bline.txt'
    stdin = StringIO()
    stdin.write(LINE1)
    stdin.write(LINE2)
    stdin.write(LINE3)
    stdin.seek(0)
    code = main(['sync', '--baseline-path', str(blpath)], stdin, StringIO())
    assert code == 0
    actual = blpath.read_text()
    line1, line2, line3 = actual.splitlines()
    assert line1 == 'views.py:0: error: Hello world  [assignment]'
    assert line2 == 'settings.py:0: error: How are you?  [union-attr]'
    assert line3 == 'python/utils.py:0: error: Second argument of Enum() must be string  [misc]'  # noqa: E501s


def test_sync_notebook(tmp_path: Path):
    blpath = tmp_path / 'bline.txt'
    stdin = StringIO()
    stdin.write(NOTEBOOK_LINE1)
    stdin.seek(0)
    code = main(['sync', '--baseline-path', str(blpath)], stdin, StringIO())
    assert code == 0
    actual = blpath.read_text()
    line1 = actual.splitlines()[0]
    assert line1 == 'fail.ipynb:cell_1:0: error: Incompatible return value type (got "int", expected "str")  [return-value]'  # noqa: E501


def test_sync_sorted(tmp_path: Path):
    blpath = tmp_path / 'bline.txt'
    stdin = StringIO()
    stdin.write(LINE1)
    stdin.write(LINE2)
    stdin.write(LINE3)
    stdin.seek(0)
    code = main(['sync', '--sort-baseline', '--baseline-path', str(blpath)], stdin, StringIO())  # noqa: E501
    assert code == 0
    actual = blpath.read_text()
    line1, line2, line3 = actual.splitlines()
    assert line1 == 'python/utils.py:0: error: Second argument of Enum() must be string  [misc]'  # noqa: E501s
    assert line2 == 'settings.py:0: error: How are you?  [union-attr]'
    assert line3 == 'views.py:0: error: Hello world  [assignment]'
